package GCData;

###################################################
#
#  Copyright 2005-2007 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;


{
    package GCItems;
    
    use XML::Parser;
    use File::Copy;
    use File::Path;
    use File::Basename;

    use GCModel;
    use GCBackend::GCBackendXmlParser;

    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {};

        $self->{parent} = $parent;

        $self->{imagesToBeRemoved} = [];
        $self->{imagesToBeAdded} = [];
        $self->{loaded} = {};

        $self->{currentItem} = -1;
        $self->{currentId} = -1;
        $self->{hasBeenDeleted} = 0;
        $self->{block} = 0;
        #$self->{filterSearch} = new GCFilterSearch;

        bless ($self, $class);
        return $self;
    }

    sub initModel
    {
        my ($self, $model) = @_;
        $self->{model} = $model;
        $self->{parent}->notifyModelChange;
    }

    sub setPanel
    {
        my ($self, $panel) = @_;
        $self->{panel} = $panel;
    }

    sub unselect
    {
        my ($self) = @_;
        
        $self->{currentItem} = -1;
    }

    sub updateCurrentFromPanel
    {
        my ($self, $panel) = @_;
        my $previousPanel = $self->{panel};
        $self->{panel} = $panel;
        $self->updateCurrent;
        $self->{panel} = $previousPanel;
    }

    sub updateCurrent
    {
        my $self = shift;

        return if $self->{currentItem} == -1;
        my $info = ($self->getItemsList)->[$self->{currentItem}];

        my $oldSortValue = $info->{$self->{currentSortField}};
        my $idField = $self->{model}->{commonFields}->{id};
        my $panelId = $self->{panel}->$idField;
        my $previousId = $info->{$idField};
        
        my $changed = 0;
        
        if ($panelId &&
           ($panelId != $previousId))
        {
		  $info->{$idField} = $self->{panel}->$idField;
		  $self->{loaded}->{information}->{maxId} = $panelId
		      if ($panelId > $self->{loaded}->{information}->{maxId});
		  $self->findMaxId if $previousId == $self->{loaded}->{information}->{maxId};
		  $self->{currentId} = $panelId;
		  $self->{toBeDisplayed}->{$panelId} = 1;
		  $self->{toBeDisplayed}->{$previousId} = 0;
		  $changed = 1;
        }
        $self->{panel}->$idField($info->{$idField}) if $self->{panel}->$idField;

        my $previous = {$idField => $previousId};

        for my $field (@{$self->{model}->{fieldsNames}})
        {
            next if $field eq $idField;
            $previous->{$field} = $info->{$field};
            next if !$self->{panel}->{$field}->hasChanged;
            $self->{panel}->{$field}->addHistory if ($self->{model}->{fieldsInfo}->{$field}->{hasHistory});
            $changed = 1;
            $info->{$field} = $self->{panel}->$field;
            $self->{parent}->{menubar}->checkFilter($field);
        }

        $self->{panel}->dataChanged($info);
        if ($changed)
        {
            my $current = $self->{parent}->{itemsView}->changeCurrent($previous, $info, $self->{currentItem});
            if ($current != $self->{currentItem})
            {
                $self->{currentItem} = $current;
                #$self->select($current, 0);
                $self->displayCurrent;
            }
            $self->{parent}->markAsUpdated;
        }
        return 0;
    }
    
    sub getTitle
    {
        my ($self, $idx) = @_;

        my $realIdx = $idx;
        $realIdx = $self->{currentItem} if $idx == undef;
        return ($self->getItemsList)->[$realIdx]->{$self->{model}->{commonFields}->{title}};
    }

    sub getCurrent
    {
        my ($self) = @_;
        return $self->{currentItem};
    }

    sub displayCurrent
    {
        my ($self) = @_;
        $self->displayInPanel($self->{panel}, undef);
    }
    
    sub displayInPanel
    {
        my ($self, $panel, $idx) = @_;
        $idx = $self->{currentItem} if $idx == undef;
        return if $self->{currentItem} < 0;
        for my $field (@{$self->{model}->{fieldsNotFormatted}})
        {
            $panel->$field(($self->getItemsList)->[$idx]->{$field});
            $panel->{$field}->resetChanged;
        }
        
        $panel->dataChanged;
    }

    sub display
    {
        my $self = shift;
        return if (! $self->{itemArray}) || (! scalar @{$self->{itemArray}});
        my $number = shift;

        $number = 0 if $number == -1;
        #$number = $self->selectToId($number);
        $self->{currentId} = ($self->getItemsList)->[$number]->{$self->{model}->{commonFields}->{id}};

        my $hasBeenReloaded = 0;

        if (($self->{currentItem} > -1) && !($self->{hasBeenDeleted}))
        {
            $hasBeenReloaded = $self->updateCurrent;
        }
        else
        {
            $self->{currentItem} = $number;
        }

        $self->{hasBeenDeleted} = 0;

        $self->{currentItem} = $number if ! $hasBeenReloaded;
        $self->displayCurrent;
        return $self->{currentItem};
    }

    sub valueToDisplayed
    {
        my ($self, $value, $field) = @_;
        return $self->{model}->getDisplayedValue($self->{model}->{fieldsInfo}->{$field}->{values}, $value);
    }
                
    sub transformValue
    {
        my ($self, $value, $field, $type) = @_;
        
        $type ||= $self->{model}->{fieldsInfo}->{$field}->{type};
        $value = $self->{parent}->transformTitle($value) if $field eq $self->{model}->{commonFields}->{title};
        $value = GCPreProcess::reverseDate($value) if $type eq 'date';
        $value = $self->valueToDisplayed($value, $field) if $type eq 'options';
        $value = GCPreProcess::multipleList($value, $type) if $type =~ /list$/o;
        return $value;
    }

    sub getValue
    {
        my ($self, $idx, $field) = @_;
        
        return ($self->getItemsList)->[$idx]->{$field};
    }
    
    sub setValue
    {
        my ($self, $idx, $field, $value) = @_;
        
        ($self->getItemsList)->[$idx]->{$field} = $value;
        if ($idx == $self->{currentItem})
        {
            $self->{panel}->$field($value);
            $self->{panel}->dataChanged;
        }
    }

    sub getItemsList
    {
        my ($self, $filter) = @_;
        
        return $self->{itemArray} if ! $filter;
        my @results = ();
        foreach (@{$self->{itemArray}})
        {
            #if ($self->{toBeDisplayed}->{$_->{$self->{model}->{commonFields}->{id}}})
            if ($self->{parent}->{filterSearch}->test($_))
            {
                push @results, $_;
            }
        }
        return \@results;
    }
    
    # Should only be used by GCCommandExecution
    sub setItemsList
    {
        my ($self, $itemsList) = @_;

        $self->{itemArray} = $itemsList;
    }

    sub getInformation
    {
        my $self = shift;
        $self->{loaded}->{information} ||= {};
        return $self->{loaded}->{information};
    }

   sub setInformation
    {
        my ($self, $info) = @_;
        $self->{loaded}->{information} = $info;
    }

    sub reloadList
    {
        my ($self, $splash, $fullProgress, $filtering) = @_;
        return if $self->{block};
        if ($splash)
        {
            $splash->initProgress if $fullProgress;
            $splash->setItemsTotal(scalar @{$self->{itemArray}})
                if $self->{itemArray};
        }
        $self->{parent}->{itemsView}->reset if $self->{parent}->{itemsView};
        
        my $lastDisplayed = -1;
        my $hasId = 0;
        my $j = 0;
        my $idField = $self->{model}->{commonFields}->{id};
        my $currentId;

        # If we don't get an history from BE, we will have to initialize it now
        my $historyNeeded = ! $self->{loaded}->{gotHistory};
        my %histories;

        foreach (@{$self->{itemArray}})
        {
            if ($historyNeeded)
            {
                foreach my $field (@{$self->{model}->{fieldsHistory}})
                {
                    #push @{$histories{$field}}, $_->{$field};
                    $self->{panel}->addHistory($_->{$field}, 1);
                }
            }

            $currentId = $_->{$idField};

            $self->{parent}->{itemsView}->addItem($_, 0);               
            $lastDisplayed = $j;
            $splash->setProgressForItemsDisplay($j) if $splash;
            $j++;
        }

        if ($historyNeeded)
        {
            for my $hfield(@{$self->{model}->{fieldsHistory}})
            {
                $self->{panel}->{$hfield}->setDropDown;
            }
            # Now we are sure we got one
            $self->{loaded}->{gotHistory} = 2;
        }

        $self->{panel}->show if $j;

        #if ($splash && $fullProgress)
        #{
        #    $splash->endProgress;
        #}

        if ($self->{currentId} == -1)
        {
            $self->{currentId}  = 0;
        }

        if (! $self->{parent}->{initializing})
        {
            $self->{parent}->reloadDone(0, $splash);
            $self->select($self->{currentItem}, 0);
        }
    }

    sub select
    {
        my ($self, $value, $init) = @_;
        return if !$self->{parent}->{itemsView};
        return $self->{parent}->{itemsView}->select($value, $init) unless  $value < -1;
    }

    sub removeCurrentItem
    {
        my $self = shift;

        my $number = $self->{currentItem};

        my $prevId = $self->{itemArray}->[$number]->{$self->{model}->{commonFields}->{id}};

        foreach (@{$self->{model}->{managedImages}})
        {
            my $image = $self->{itemArray}->[$number]->{$_};
            my $imagePrefix = $self->{parent}->{imagePrefix};
            if ($image =~ /(\/|\\)$imagePrefix[0-9]*\./)
            {
                $self->markToBeRemoved($image);
            }
        }

        my $oldInfo = splice @{$self->{itemArray}}, $number, 1;

        # Synchronizing list of displayed
#        my $i = scalar @{$self->{selectToIdArray}} - 1;
#        my $after = 1;
#        my $listItem;
#        while ($i >= 0)
#        {
#            if ($self->{selectToIdArray}->[$i] == $number)
#            {
#                splice @{$self->{selectToIdArray}}, $i, 1;
#                $listItem = $i;
#                $after = 0;
#            }
#            elsif ($after)
#            {
#                $self->{selectToIdArray}->[$i]--;
#            }
#            $i--;
#        }

        my $newIdx = $self->{parent}->{itemsView}->removeItem($number, $prevId, $oldInfo);
        $self->{currentId} = $self->{itemArray}->[$newIdx]->{$self->{model}->{commonFields}->{id}};

        $self->{currentItem} = $newIdx;
        $self->{hasBeenDeleted} = 1;
        $self->displayCurrent;
    }

    sub addItem
    {
        my ($self, $info, $keepId, $noSelect) = @_;
        my $nbItems = scalar @{$self->{itemArray}};
#        $self->{panel}->show if ! $nbItems;

        my $currentId;
        if ($keepId)
        {
            $currentId = $self->{itemArray}->[$nbItems]->{$self->{model}->{commonFields}->{id}};
        }
        else
        {
            $self->{loaded}->{information}->{maxId}++;        
            $currentId = $self->{loaded}->{information}->{maxId};
            $self->{itemArray}->[$nbItems]->{$self->{model}->{commonFields}->{id}} = $currentId;
        }
        
        for my $field (@{$self->{model}->{fieldsNames}})
        {
            next if $field eq $self->{model}->{commonFields}->{id};
            if ($self->{model}->{fieldsInfo}->{$field}->{hasHistory})
            {
                $self->{panel}->{$field}->addHistory($info->{$field});
            }
            $self->{itemArray}->[$nbItems]->{$field} = $info->{$field};
        }

        $self->{currentId} = $self->{loaded}->{information}->{maxId};
#        $self->{toBeDisplayed}->{$self->{currentId}} = 1;

        $self->{parent}->{itemsView}->addItem($self->{itemArray}->[$nbItems], 1);

        $self->{currentItem} = $nbItems;

        #push @{$self->{selectToIdArray}}, $nbItems;

        $self->select($nbItems, 0)
            if !$noSelect;
        
        $self->{parent}->{itemsView}->showCurrent;
        
        return $currentId;
    }
    
    sub setOptions
    {
        my ($self, $options) = @_;

        $self->{options} = $options;

        #return $self->load($options->file, $splash, 0);
    }

    sub markToBeRemoved
    {
        my ($self, $image) = @_;

        push @{$self->{imagesToBeRemoved}}, $image;
    }

    sub markToBeAdded
    {
        my ($self, $image) = @_;
        push @{$self->{imagesToBeAdded}}, $image;        
    }
    
    sub removeMarkedPictures
    {
        my $self = shift;
        my $image;
        foreach $image(@{$self->{imagesToBeRemoved}})
        {
            unlink $image;
        }

        $self->{imagesToBeRemoved} = [];
    }

    sub addMarkedPictures
    {
        my $self = shift;

        $self->{imagesToBeAdded} = [];
    }
    
    sub clean
    {
        my $self = shift;
        my $image;
        foreach (@{$self->{imagesToBeAdded}})
        {
            unlink $_;
        }
        $self->{oldImagesDirectory} = {};
        $self->{newImagesDirectory} = undef;
        $self->{copyImagesWhenChangingDir} = 0;
    }

    sub setNewImagesDirectory
    {
        # $prev is also a parameter because we didn't store it here
        my ($self, $new, $prev, $withCopy) = @_;
        $new =~ s|/$||;
        $self->{newImagesDirectory} = $new;
        $self->{copyImagesWhenChangingDir} = $withCopy;
        # We stored the previous one as a hash so it will be easier for tests
        $prev =~ s|/$||;
        $self->{oldImagesDirectory}->{$prev} = 1;
    }

    sub queryReplace
    {
        my ($self, $field, $old, $new) = @_;
        foreach (@{$self->{itemArray}})
        {
            $_->{$field} =~ s/$old/$new/gi;
        }
        $self->displayCurrent;
        $self->reloadList;
    }
    
    sub findMaxId
    {
        my $self = shift;

        $self->{loaded}->{information}->{maxId} = -1;
        foreach (@{$self->{itemArray}})
        {
            $self->{loaded}->{information}->{maxId} = $_->{$self->{model}->{commonFields}->{id}}
                if $_->{$self->{model}->{commonFields}->{id}} > $self->{loaded}->{information}->{maxId};
        }
    }
    
    sub clearList
    {
        my $self = shift;
        $self->{currentItem} = -1;
        $self->{loaded} = {};
        $self->{itemArray} = [];
        #$self->{selectToIdArray} = [];
        $self->{currentId} = -1;
        $self->{toBeDisplayed} = {};
        $self->{panel}->hide if $self->{panel};
        $self->{parent}->{itemsView}->clearCache if $self->{parent}->{itemsView};
        $self->{parent}->{itemsView}->reset if $self->{parent}->{itemsView};
        #$self->{parent}->reloadDone(1) if ! $self->{parent}->{initializing};
        #$self->reloadList if ! $self->{parent}->{initializing};
    }

    sub getNbItems
    {
        my $self = shift;
        return 0 if ! $self->{itemArray};
        return scalar @{$self->{itemArray}};
    }

    sub setLock
    {
        my ($self, $value) = @_;
        $self->{loaded}->{information}->{locked} = $value;
    }

    sub getLock
    {
        my $self = shift;
        return $self->{loaded}->{information}->{locked};
    }

    my $globalInstance;

    sub load
    {
        my ($self, $file, $splash, $fullProgress, $noReload) = @_;

        my $initTime;
        if ($ENV{GCS_PROFILING} > 0)
        {
            eval 'use Time::HiRes';
            eval '$initTime = [Time::HiRes::gettimeofday()]';
        }

        $self->clean;
        if (!$file)
        {
            $self->{parent}->setCurrentModel;
            return 0;
        }

        my $collection;

        $self->{block} = 1;
        $self->clearList;
        $self->{block} = 0;
        $self->{splash} = $splash;

        eval
        {
            $self->{backend} = new GCBackend::GCBeXmlParser($self->{parent});
            
            $self->{backend}->setParameters($file);
            
            $self->{loaded} = $self->{backend}->load($splash);
            # We keep a direct access to this one
            $self->{itemArray} = $self->{loaded}->{data};
        };
        if ($@)
        {
            my @error = ('Fatal error while reading file', $@);
            return (0, \@error);
        }
        elsif ($self->{loaded}->{error})
        {
            return (0, $self->{loaded}->{error});
        }

        # Hide the panel if no item
        if (! scalar @{$self->{itemArray}})
        {
            $self->{panel}->hide;
        }

        # gotHistory = 1 means we got one but it has not been set in components
        if ($self->{loaded}->{gotHistory} == 1)
        {
            for my $hfield(@{$self->{model}->{fieldsHistory}})
            {
                if (exists $self->{loaded}->{histories}->{$hfield})
                {
                    $self->{panel}->{$hfield}->setValues($self->{loaded}->{histories}->{$hfield}, 1);
                }
                $self->{panel}->{$hfield}->setDropDown;
            }
#            $self->{loaded}->{gotHistory} = 2;
        }
        elsif ($self->{loaded}->{gotHistory} == 2)
        {
            for my $hfield(@{$self->{model}->{fieldsHistory}})
            {
                $self->{panel}->{$hfield}->setDropDown;
            }
        }

        $self->reloadList($splash, $fullProgress) unless $noReload;
                
#        $self->{parent}->refreshFilters;

        if ($ENV{GCS_PROFILING} > 0)
        {
            my $elapsed;
            eval '$elapsed = Time::HiRes::tv_interval($initTime)';
            print "Load time : $elapsed\n";
        }

        return 1;
    }

    sub movePictures
    {
        my ($self) = @_;
        
        eval {
            mkpath $self->{newImagesDirectory};
            my $file;
            foreach (@{$self->getItemsList})
            {
                foreach my $pic(@{$self->{model}->{fieldsImage}})
                {
                    $file = GCUtils::getDisplayedImage($_->{$pic},
                                                       '',
                                                       $self->{options}->file);
                    # Not moving picture if it is not in the previous directory
                    next if !$file;
                    next if ! exists $self->{oldImagesDirectory}->{Cwd::realpath(dirname($file))};
                    (my $suffix = $file) =~ s/.*?(\.[^.]*)$/$1/;
                    my $new = 
                        $self->{parent}->getUniqueImageFileName(
                            $suffix, 
                            $_->{$self->{model}->{commonFields}->{title}});
                    
                    if ($self->{copyImagesWhenChangingDir})
                    {
                        copy $file, $new;
                    }
                    else
                    {
                        move $file, $new;
                    }
                    $_->{$pic} = $new;
                }
            }
        };
        return $@ if $@;
        $self->displayCurrent;
        $self->{newImagesDirectory} = undef;
        $self->{copyImagesWhenChangingDir} = 0;
        $self->{oldImagesDirectory} = {};
        return 0;
    }

    sub save
    {
        my ($self, $splash) = @_;

        my $initTime;
        if ($ENV{GCS_PROFILING} > 0)
        {
            eval 'use Time::HiRes';
            eval '$initTime = [Time::HiRes::gettimeofday()]';
        }

        $self->updateCurrent if ($self->{currentItem} > -1);

        # TODO : Use progress bar for this operation also
        my $moveError = $self->movePictures
            if $self->{newImagesDirectory};
        return (0, ['SaveError', $moveError])
            if $moveError;
        

        if ($splash)
        {
            $splash->initProgress($self->{parent}->{lang}->{StatusSave});
            $splash->setItemsTotal(scalar @{$self->{itemArray}});
        }
        $self->addMarkedPictures;
        $self->removeMarkedPictures;

        # We re-generate histories to give it to backend
        # Deactivated for the moment
#        if ($self->{panel})
#        {
#            my %histories;
#            for my $hfield(@{$self->{model}->{fieldsHistory}})
#            {
#                $histories{$hfield} = $self->{panel}->{$hfield}->getValues;
#            }
#            $self->{backend}->setHistories(\%histories);
#        }

        $self->{backend} = new GCBackend::GCBeXmlParser($self->{parent})
            if !$self->{backend};
        
        $self->{backend}->setParameters($self->{options}->file);

        my $result = $self->{backend}->save($self->{itemArray},
                                            $self->{loaded}->{information},
                                            $splash);

        $self->{parent}->endProgress;
        if ($result->{error})
        {
            return (0, $result->{error});
        }
        $self->{parent}->removeUpdatedMark;

        if ($ENV{GCS_PROFILING} > 0)
        {
            my $elapsed;
            eval '$elapsed = Time::HiRes::tv_interval($initTime)';
            print "Save time : $elapsed\n";
        }
        return 1;
    }

    
}

1;
